/*
 * File:   bm83.h
 * Author: Ted Salmon <tass2001@gmail.com>
 * Description:
 *     Implementation of the Microchip BM83 Bluetooth UART API
 */
#ifndef BM83_H
#define BM83_H
#include <stdint.h>
#include "bt_common.h"

extern int8_t BTBM83MicGainTable[];

#define BM83_AVRCP_DATA_PLAYBACK_STATUS_PLAYING 0x01
#define BM83_AVRCP_DATA_PLAYBACK_STATUS_PAUSED 0x02
#define BM83_AVRCP_DATA_PLAYBACK_STATUS_NOW_PLAYING_CHANGED 0x09
#define BM83_AVRCP_DATA_ELEMENT_TYPE_TITLE 0x01
#define BM83_AVRCP_DATA_ELEMENT_TYPE_ARTIST 0x02
#define BM83_AVRCP_DATA_ELEMENT_TYPE_ALBUM 0x03
#define BM83_AVRCP_DATA_CAP_TYPE_EVENTS 0x03

#define BM83_AVRCP_EVT_PLAYBACK_STATUS_CHANGED 0x01
#define BM83_AVRCP_EVT_PLAYBACK_TRACK_CHANGED 0x02
#define BM83_AVRCP_EVT_TRACK_REACHED_END 0x03
#define BM83_AVRCP_EVT_TRACK_REACHED_START 0x04
#define BM83_AVRCP_EVT_PLAYBACK_POS_CHANGED 0x05
#define BM83_AVRCP_EVT_NOW_PLAYING_CONTENT_CHANGED 0x09
#define BM83_AVRCP_EVT_ADDRESSED_PLAYER_CHANGED 0x0B
#define BM83_AVRCP_EVT_VOLUME_CHANGED 0x0D

#define BM83_AVRCP_PDU_GET_CAPABILITIES 0x10
#define BM83_AVRCP_PDU_GET_ELEMENT_ATTRIBUTES 0x20
#define BM83_AVRCP_PDU_GET_PLAY_STATUS 0x30
#define BM83_AVRCP_PDU_NOTIFICATION 0x31

#define BM83_LINKED_DEVICE_QUERY_NAME 0x00


#define BM83_FRAME_DB0 0
#define BM83_FRAME_DB1 1
#define BM83_FRAME_DB2 2
#define BM83_FRAME_DB3 3
#define BM83_FRAME_DB4 4
#define BM83_FRAME_DB5 5
#define BM83_FRAME_DB6 6
#define BM83_FRAME_DB7 7
#define BM83_FRAME_DB8 8
#define BM83_FRAME_DB9 9
#define BM83_FRAME_DB10 10
#define BM83_FRAME_DB11 11
#define BM83_FRAME_DB12 12
#define BM83_FRAME_DB13 13

#define BM83_FRAME_SIZE_MIN 0x05
#define BM83_FRAME_CTRL_BYTE_COUNT 0x04

#define BM83_OFFSET_EVENT_CODE 0x03
#define BM83_OFFSET_EVENT_DATA 0x04

#define BM83_CMD_MAKE_CALL 0x00
#define BM83_CMD_MAKE_EXTENSION_CALL 0x01
#define BM83_CMD_MMI_ACTION 0x02
#define BM83_CMD_EVENT_MASK_SETTING 0x03
#define BM83_CMD_MUSIC_CONTROL 0x04
#define BM83_CMD_CHANGE_DEVICE_NAME 0x05
#define BM83_CMD_CHANGE_PIN_CODE 0x06
#define BM83_CMD_BTM_PARAMETER_SETTING 0x07
#define BM83_CMD_READ_BTM_VERSION 0x08
#define BM83_CMD_VENDOR_AT_COMMAND 0x0A
#define BM83_CMD_AVC_VENDOR_DEPENDENT_CMD 0x0B
#define BM83_CMD_AVC_GROUP_NAVIGATION 0x0C
#define BM83_CMD_READ_LINK_STATUS 0x0D
#define BM83_CMD_READ_PAIRED_DEVICE_RECORD 0x0E
#define BM83_CMD_READ_LOCAL_BD_ADDRESS 0x0F
#define BM83_CMD_READ_LOCAL_DEVICE_NAME 0x10
#define BM83_CMD_SEND_SPP_IAP_OR_LE_DATA 0x12
#define BM83_CMD_BTM_UTILITY_FUNCTION 0x13
#define BM83_CMD_EVENT_ACK 0x14
#define BM83_CMD_ADDITIONAL_PROFILES_LINK_SETUP 0x15
#define BM83_CMD_READ_LINKED_DEVICE_INFORMATION 0x16
#define BM83_CMD_PROFILES_LINK_BACK 0x17
#define BM83_CMD_DISCONNECT 0x18
#define BM83_CMD_MCU_STATUS_INDICATION 0x19
#define BM83_CMD_USER_CONFIRM_SPP_REQ_REPLY 0x1A
#define BM83_CMD_SET_HF_SPEAKER_GAIN_LEVEL 0x1B
#define BM83_CMD_EQ_MODE_SETTING 0x1C
#define BM83_CMD_DSP_NR_CTRL 0x1D
#define BM83_CMD_GPIO_CONTROL 0x1E
#define BM83_CMD_MCU_UART_RX_BUFFER_SIZE 0x1F
#define BM83_CMD_VOICE_PROMPT_CMD 0x20
#define BM83_CMD_SET_OVERALL_GAIN 0x23
#define BM83_CMD_READ_BTM_SETTING 0x24
#define BM83_CMD_READ_BTM_BATTERY_CHARGE_STATUS 0x25
#define BM83_CMD_MCU_UPDATE_CMD 0x26
#define BM83_CMD_REPORT_BATTERY_CAPACITY 0x27
#define BM83_CMD_LE_ANCS_SERVICE_CMD 0x28
#define BM83_CMD_LE_SIGNALING_CMD 0x29
#define BM83_CMD_MSPK_VENDOR_CMD 0x2A
#define BM83_CMD_READ_MSPK_LINK_STATUS 0x2B
#define BM83_CMD_MSPK_SYNC_AUDIO_EFFECT 0x2C
#define BM83_CMD_LE_GATT_CMD 0x2D
#define BM83_CMD_LE_APP_CMD 0x2F
#define BM83_CMD_DSP_RUNTIME_PROGRAM 0x30
#define BM83_CMD_READ_VENDOR_STORED_DATA 0x31
#define BM83_CMD_READ_IC_VERSION_IINFO 0x32
#define BM83_CMD_READ_BTM_LINK_MODE 0x34
#define BM83_CMD_CONFIGURE_VENDOR_PARAMETER 0x35
#define BM83_CMD_MSPK_EXCHANGE_LINK_INFO_CMD 0x37
#define BM83_CMD_MSPK_SET_GIAC 0x38
#define BM83_CMD_READ_FEATURE_LIST 0x39
#define BM83_CMD_PERSONAL_MSPK_GROUP_CONTROL 0x3A
#define BM83_CMD_TEST_DEVICE 0x3B
#define BM83_CMD_READ_EEPROM_DATA 0x3C
#define BM83_CMD_WRITE_EEPROM_DATA 0x3D
#define BM83_CMD_LE_SIGNALING2_CMD 0x3E
#define BM83_CMD_PBAPC_CMD 0x3F
#define BM83_CMD_TWS_CMD 0x40
#define BM83_CMD_AVRCP_BROWSING_CMD 0x41
#define BM83_CMD_READ_PAIRED_LINK_KEY_INFO 0x42
#define BM83_CMD_AUDIO_TRANSCEIVER_CMD 0x44
#define BM83_CMD_BUTTON_MMI_SETTING_CMD 0x46
#define BM83_CMD_BUTTON_OPERATION_CMD 0x47
#define BM83_CMD_READ_BUTTON_MMI_SETTING_CMD 0x48
#define BM83_CMD_DFU 0x49
#define BM83_CMD_AVRCP_VENDOR_DEPENDENT_CMD 0x4A
#define BM83_CMD_CONCERT_MODE_ENDLESS_GROUPING 0x4B
#define BM83_CMD_READ_RUNTIME_LATENCY 0x4C
#define BM83_CMD_TOGGLE_AUDIO_SOURCE 0xCC

#define BM83_CMD_ACTION_STOP_FF_RW 0x00
#define BM83_CMD_ACTION_FF 0x01
#define BM83_CMD_ACTION_RW 0x03
#define BM83_CMD_ACTION_PLAY 0x05
#define BM83_CMD_ACTION_PAUSE 0x06
#define BM83_CMD_ACTION_STOP 0x08
#define BM83_CMD_ACTION_NEXT 0x09
#define BM83_CMD_ACTION_PREVIOUS 0x0A

#define BM83_CMD_BTM_FUNCTION_DISCO_CONN 0x03
#define BM83_CMD_BTM_FUNCTION_PARAM_NO_CONN 0x00
#define BM83_CMD_BTM_FUNCTION_PARAM_CONN 0x01

#define BM83_CMD_DISCONNECT_PARAM_ALL 0x1F
#define BM83_CMD_DISCONNECT_PARAM_HF 0x02

#define BM83_CMD_MMI_ACTION_ACCEPT_CALL 0x04
#define BM83_CMD_MMI_ACTION_END_CALL 0x06
#define BM83_CMD_MMI_ACTION_VR_OPEN 0x0A
#define BM83_CMD_MMI_ACTION_VR_CLOSE 0x0B
#define BM83_CMD_MMI_ACTION_REDIAL 0x0C
#define BM83_CMD_MMI_ACTION_MIC_GAIN_UP 0x24
#define BM83_CMD_MMI_ACTION_MIC_GAIN_DOWN 0x25
#define BM83_CMD_MMI_ACTION_RESTORE 0x56

#define BM83_DATA_BTM_STATUS_POWER_OFF 0x00
#define BM83_DATA_BTM_STATUS_PAIRING_ON 0x01
#define BM83_DATA_BTM_STATUS_POWER_ON 0x02
#define BM83_DATA_BTM_STATUS_PAIRING_OK 0x03
#define BM83_DATA_BTM_STATUS_PAIRING_NOK 0x04
#define BM83_DATA_BTM_STATUS_HFP_CONN 0x05
#define BM83_DATA_BTM_STATUS_A2DP_CONN 0x06
#define BM83_DATA_BTM_STATUS_HFP_DISCO 0x07
#define BM83_DATA_BTM_STATUS_A2DP_DISCO 0x08
#define BM83_DATA_BTM_STATUS_SCO_CONN 0x09
#define BM83_DATA_BTM_STATUS_SCO_DISCO 0x0A
#define BM83_DATA_BTM_STATUS_AVRCP_CONN 0x0B
#define BM83_DATA_BTM_STATUS_AVRCP_DISCO 0x0C
#define BM83_DATA_BTM_STATUS_STANDARD_SPP_CONN 0x0D
#define BM83_DATA_BTM_STATUS_STANDARD_SPP_IAP_DISCO 0x0E
#define BM83_DATA_BTM_STATUS_STANDBY_ON 0x0F
#define BM83_DATA_BTM_STATUS_IAP_CONN 0x10
#define BM83_DATA_BTM_STATUS_ACL_DISCO 0x11
#define BM83_DATA_BTM_STATUS_MAP_CONN 0x12
#define BM83_DATA_BTM_STATUS_MAP_OPERATION_FORBIDDEN 0x13
#define BM83_DATA_BTM_STATUS_MAP_DISCONN 0x14
#define BM83_DATA_BTM_STATUS_ACL_CONN 0x15
#define BM83_DATA_BTM_STATUS_SPP_IAP_DISCONN_NO_OTHER_PROFILE 0x16
#define BM83_DATA_BTM_STATUS_LINK_BACK_ACL 0x17
#define BM83_DATA_BTM_STATUS_INQUIRY_ON 0x18
#define BM83_DATA_BTM_STATUS_AUDIO_SRC_NOT_AUX_NOT_A2DP 0x80
#define BM83_DATA_BTM_STATUS_AUDIO_SRC_AUX_IN 0x81
#define BM83_DATA_BTM_STATUS_AUDIO_SRC_A2DP 0x82

#define BM83_DATA_CALL_STATUS_IDLE 0x00
#define BM83_DATA_CALL_STATUS_VR 0x01
#define BM83_DATA_CALL_STATUS_INCOMING 0x02
#define BM83_DATA_CALL_STATUS_OUTGOING 0x03
#define BM83_DATA_CALL_STATUS_ACTIVE 0x04
#define BM83_DATA_CALL_STATUS_ACTIVE_CALL_WAITING 0x05
#define BM83_DATA_CALL_STATUS_ACTIVE_CALL_HOLD 0x06

#define BM83_DATA_LINK_STATE_POWER_OFF 0x00
#define BM83_DATA_LINK_STATE_PAIRING_ON 0x01
#define BM83_DATA_LINK_STATE_STANDBY 0x02
#define BM83_DATA_LINK_STATE_HF_CONNECTED_ONLY 0x03
#define BM83_DATA_LINK_STATE_A2DP_CONNECTED_ONLY 0x04
#define BM83_DATA_LINK_STATE_SPP_CONNECTED_ONLY 0x05
#define BM83_DATA_LINK_STATE_MULTI_PROFILES_CONNECTED 0x06

#define BM83_DATA_BTM_INITIAL_STATUS_BOOT_COMPLETE 0x00

#define BM83_DATA_AVC_RSP_NOT_IMPL 0x08
#define BM83_DATA_AVC_RSP_ACCEPT 0x09
#define BM83_DATA_AVC_RSP_REJECT 0x0A
#define BM83_DATA_AVC_RSP_STABLE 0x0C
#define BM83_DATA_AVC_RSP_CHANGED 0x0D
#define BM83_DATA_AVC_RSP_INTERIM 0x0F

#define BM83_DATA_LINK_BACK_LAST_DEVICE 0x00
#define BM83_DATA_LINK_BACK_MAC_ID 0x05
#define BM83_DATA_LINK_BACK_PROFILES_A2DP 0x04
#define BM83_DATA_LINK_BACK_PROFILES_HF 0x02
#define BM83_DATA_LINK_BACK_PROFILES_A2DP_HF 0x06
#define BM83_DATA_LINK_BACK_ACL 0x00
#define BM83_DATA_LINK_BACK_HF 0x01
#define BM83_DATA_LINK_BACK_A2DP 0x02
#define BM83_DATA_LINK_BACK_ACL_FAILED 0xFF
#define BM83_DATA_LINK_BACK_A2DP_HF_SUCCESS 0x00
#define BM83_DATA_LINK_BACK_A2DP_HF_FAILED 0x01

#define BM83_DATA_DSP_REPORTED_SR_8kHz 0x00
#define BM83_DATA_DSP_REPORTED_SR_16kHz 0x02
#define BM83_DATA_DSP_REPORTED_SR_32kHz 0x04
#define BM83_DATA_DSP_REPORTED_SR_48kHz 0x05
#define BM83_DATA_DSP_REPORTED_SR_44_1kHz 0x06
#define BM83_DATA_DSP_REPORTED_SR_88kHz 0x07
#define BM83_DATA_DSP_REPORTED_SR_96kHz 0x08

#define BM83_DATA_DSP_REPORTED_MODE_NONE 0x00
#define BM83_DATA_DSP_REPORTED_MODE_PREPARE 0x01
#define BM83_DATA_DSP_REPORTED_MODE_AUDIO_IN 0x02
#define BM83_DATA_DSP_REPORTED_MODE_PCM 0x03
#define BM83_DATA_DSP_REPORTED_MODE_A2DP 0x04
#define BM83_DATA_DSP_REPORTED_MODE_SCO 0x06
#define BM83_DATA_DSP_REPORTED_MODE_TONE 0x07
#define BM83_DATA_DSP_REPORTED_MODE_VP 0x08

#define BM83_DATA_BOOT_STATUS_POWER_ON 0x01


#define BM83_EVT_COMMAND_ACK 0x00
#define BM83_EVT_BTM_STATUS 0x01
#define BM83_EVT_CALL_STATUS 0x02
#define BM83_EVT_CALLER_ID 0x03
#define BM83_EVT_SMS_RECEIVED_INDICATION 0x04
#define BM83_EVT_MISSED_CALL_INDICATION 0x05
#define BM83_EVT_PHONE_MAX_BATTERY_LEVEL 0x06
#define BM83_EVT_PHONE_CURRENT_BATTERY_LEVEL 0x07
#define BM83_EVT_ROAMING_STATUS 0x08
#define BM83_EVT_PHONE_MAX_SIGNAL_STRENGTH_LEVEL 0x09
#define BM83_EVT_PHONE_CURRENT_SIGNAL_STRENGTH_LEVEL 0x0A
#define BM83_EVT_PHONE_SERVICE_STATUS 0x0B
#define BM83_EVT_BTM_BATTERY_STATUS 0x0C
#define BM83_EVT_BTM_CHARGING_STATUS 0x0D
#define BM83_EVT_RESET_TO_DEFAULT 0x0E
#define BM83_EVT_REPORT_HF_GAIN_LEVEL 0x0F
#define BM83_EVT_EQ_MODE_INDICATION 0x10
#define BM83_EVT_READ_LINKED_DEVICE_INFORMATION_REPLY 0x17
#define BM83_EVT_READ_BTM_VERSION_REPLY 0x18
#define BM83_EVT_CALL_LIST_REPORT 0x19
#define BM83_EVT_AVC_SPECIFIC_RSP 0x1A
#define BM83_EVT_BTM_UTILITY_REQ 0x1B
#define BM83_EVT_VENDOR_AT_CMD_RSP 0x1C
#define BM83_EVT_REPORT_VENDOR_AT_EVENT 0x1D
#define BM83_EVT_READ_LINK_STATUS_REPLY 0x1E
#define BM83_EVT_READ_PAIRED_DEVICE_RECORD_REPLY 0x1F
#define BM83_EVT_READ_LOCAL_BD_ADDRESS_REPLY 0x20
#define BM83_EVT_READ_LOCAL_DEVICE_NAME_REPLY 0x21
#define BM83_EVT_REPORT_SPP_IAP_DATA 0x22
#define BM83_EVT_REPORT_LINK_BACK_STATUS 0x23
#define BM83_EVT_REPORT_RING_TONE_STATUS 0x24
#define BM83_EVT_USER_CONFRIM_SSP_REQ 0x25
#define BM83_EVT_REPORT_AVRCP_VOL_CTRL 0x26
#define BM83_EVT_REPORT_INPUT_SIGNAL_LEVEL 0x27
#define BM83_EVT_REPORT_IAP_INFO 0x28
#define BM83_EVT_REPORT_AVRCP_ABS_VOL_LEVEL 0x29
#define BM83_EVT_REPORT_VOICE_PROMPT_STATUS 0x2A
#define BM83_EVT_REPORT_TYPE_CODEC 0x2D
#define BM83_EVT_REPORT_TYPE_BTM_SETTING 0x2E
#define BM83_EVT_REPORT_MCU_UPDATE_REPLY 0x2F
#define BM83_EVT_REPORT_BTM_INITIAL_STATUS 0x30
#define BM83_EVT_LE_ANCS_SERVICE_EVENT 0x31
#define BM83_EVT_LE_SIGNALING_EVENT 0x32
#define BM83_EVT_REPORT_MSPK_LINK_STATUS 0x33
#define BM83_EVT_REPORT_MSPK_VENDOR_EVENT 0x34
#define BM83_EVT_REPORT_MSPK_AUDIO_SETTING 0x35
#define BM83_EVT_REPORT_SOUND_EFFECT_STATUS 0x36
#define BM83_EVT_REPORT_VENDOR_STORED_DATA 0x37
#define BM83_EVT_REPORT_IC_VERSION_INFO 0x38
#define BM83_EVT_REPORT_LE_GATT_EVENT 0x39
#define BM83_EVT_REPORT_BTM_LINK_MODE 0x3A
#define BM83_EVT_REPORT_MSPK_EXCHANGE_LINK_INFO 0x3D
#define BM83_EVT_REPORT 0x3E
#define BM83_EVT_REPORT_CSB_CLK 0x3F
#define BM83_EVT_REPORT_READ_FEATURE_LIST_REPLY 0x40
#define BM83_EVT_REPORT_TEST_RESULT_REPLY 0x41
#define BM83_EVT_REPORT_READ_EEPROM_DATA 0x42
#define BM83_EVT_PBAPC_EVENT 0x43
#define BM83_EVT_AVRCP_BROWSING_EVENT 0x44
#define BM83_EVT_REPORT_PAIRED_LINK_KEY_INFO 0x45
#define BM83_EVT_REPORT_TWS_RX_VENDOR_EVENT 0x53
#define BM83_EVT_REPORT_TWS_LOCAL_DEVICE_STATUS 0x54
#define BM83_EVT_REPORT_TWS_VAD_DATA 0x55
#define BM83_EVT_REPORT_TWS_RADIO_CONDITION 0x56
#define BM83_EVT_REPORT_TWS_EAR_BUD_POSITION 0x57
#define BM83_EVT_REPORT_TWS_SECONDARY_DEVICE_STATUS 0x58
#define BM83_EVT_AUDIO 0x5A
#define BM83_EVT_READ_BUTTON_MMI_SETTING_REPLY 0x5C
#define BM83_EVT_AVRCP_VENDOR_DEPENDENT_RSP 0x5D
#define BM83_EVT_RUNTIME 0x5E

#define BM83_UART_START_WORD 0xAA

/* Define commands */
void BM83CommandAVRCPGetCapabilities(BT_t *);
void BM83CommandAVRCPGetElementAttributesAll(BT_t *);
void BM83CommandAVRCPRegisterNotification(BT_t *, uint8_t);
void BM83CommandBTMUtilityFunction(BT_t *, uint8_t, uint8_t);
void BM83CommandCallAccept(BT_t *);
void BM83CommandCallEnd(BT_t *);
void BM83CommandConnect(BT_t *, BTPairedDevice_t *, uint8_t);
void BM83CommandDeviceDiscovery(BT_t *);
void BM83CommandDial(BT_t *, char *);
void BM83CommandDisconnect(BT_t *, uint8_t);
void BM83CommandLinkBackLastDevice(BT_t *);
void BM83CommandMicGainDown(BT_t *);
void BM83CommandMicGainUp(BT_t *);
void BM83CommandMusicControl(BT_t *, uint8_t);
void BM83CommandPairingEnable(BT_t *);
void BM83CommandPairingDisable(BT_t *);
void BM83CommandPowerOn(BT_t *);
void BM83CommandReadLinkStatus(BT_t *);
void BM83CommandReadLinkedDeviceInformation(BT_t *, uint8_t);
void BM83CommandReadLocalBDAddress(BT_t *);
void BM83CommandReadPairedDevices(BT_t *);
void BM83CommandRedial(BT_t *);
void BM83CommandRestore(BT_t *);
void BM83CommandVendorATCommand(BT_t *, char *);
void BM83CommandVoiceRecognitionClose(BT_t *);
void BM83CommandVoiceRecognitionOpen(BT_t *);
/* Process Events */
void BM83ProcessEventAVCSpecificRsp(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventAVCVendorDependentRsp(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventBTMStatus(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventCallStatus(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventCallerID(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventReadLinkStatus(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventReadLinkedDeviceInformation(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventReadPairedDeviceRecord(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventReportLinkBackStatus(BT_t *, uint8_t *, uint16_t);
void BM83ProcessEventReportTypeCodec(BT_t *, uint8_t *, uint16_t );
void BM83ProcessDataGetAllAttributes(BT_t *, uint8_t *, uint8_t, uint16_t);
/* RX / TX */
void BM83Process(BT_t *);
void BM83SendCommand(BT_t *, uint8_t *, size_t);

#endif /* BM83_H */
