/*
 * File:   config.h
 * Author: Ted Salmon <tass2001@gmail.com>
 * Description: 
 *     Get & Set Configuration items on the EEPROM
 */
#ifndef CONFIG_H
#define CONFIG_H
#include "eeprom.h"

/* EEPROM 0x00 - 0x07: Reserved for the BlueBus */
#define CONFIG_SN_ADDRESS_MSB 0x00
#define CONFIG_SN_ADDRESS_LSB 0x01
#define CONFIG_FIRMWARE_VERSION_MAJOR_ADDRESS 0x02
#define CONFIG_FIRMWARE_VERSION_MINOR_ADDRESS 0x03
#define CONFIG_FIRMWARE_VERSION_PATCH_ADDRESS 0x04
#define CONFIG_BUILD_DATE_ADDRESS_WEEK 0x05
#define CONFIG_BUILD_DATE_ADDRESS_YEAR 0x06
#define CONFIG_BOOTLOADER_MODE_ADDRESS 0x07
/* EEPROM 0x08 - 0x0E: Reserved for Trap counters */
#define CONFIG_TRAP_OSC 0x08
#define CONFIG_TRAP_ADDR 0x09
#define CONFIG_TRAP_STACK 0x0A
#define CONFIG_TRAP_MATH 0x0B
#define CONFIG_TRAP_NVM 0x0C
#define CONFIG_TRAP_GEN 0x0D
#define CONFIG_TRAP_LAST_ERR 0x0E
/* EEPROM 0x0F - 0x19: I-Bus / Vehicle Data */
#define CONFIG_UI_MODE_ADDRESS 0x0F
#define CONFIG_NAV_TYPE_ADDRESS 0x10
#define CONFIG_VEHICLE_TYPE_ADDRESS 0x11
#define CONFIG_VEHICLE_VIN_ADDRESS {0x12, 0x13, 0x14, 0x15, 0x16}
#define CONFIG_LM_VARIANT_ADDRESS 0x17
/* EEPROM 0x1A - 0x50: User Configurable Settings */
/* Config 0x1A - 0x24: UI Settings */
#define CONFIG_SETTING_METADATA_MODE_ADDRESS 0x1D
#define CONFIG_SETTING_BMBT_DEFAULT_MENU_ADDRESS 0x1E
#define CONFIG_SETTING_BMBT_DASHBOARD_OBC_ADDRESS 0x1F
#define CONFIG_SETTING_BMBT_TEMP_DISPLAY_ADDRESS 0x20
#define CONFIG_SETTING_LANGUAGE_ADDRESS 0x21
#define CONFIG_SETTING_MONITOR_OFF_ADDRESS 0x22
#define CONFIG_SETTING_BMBT_DIST_UNIT_ADDRESS 0x23
/* Config 0x25 - 0x35: Comfort Settings */
#define CONFIG_SETTING_COMFORT_BLINKERS_ADDRESS 0x25
#define CONFIG_SETTING_COMFORT_LOCKS_ADDRESS 0x26
#define CONFIG_SETTING_COMFORT_MIRRORS_ADDRESS 0x27
#define CONFIG_SETTING_COMFORT_WELCOME_LIGHTS_ADDRESS 0x28
#define CONFIG_SETTING_COMFORT_HOME_LIGHTS_ADDRESS 0x29
#define CONFIG_SETTING_COMFORT_PARKING_LAMPS_ADDRESS 0x2A
#define CONFIG_SETTING_COMFORT_DATE_TIME_ADDRESS 0x2B
#define CONFIG_SETTING_COMFORT_AUTOZOOM_ADDRESS 0x2C
#define CONFIG_SETTING_COMFORT_PDC_ADDRESS 0x2D
/* Config 0x36 - 0x3F: Telephony Settings */
#define CONFIG_SETTING_HFP_ADDRESS 0x36
#define CONFIG_SETTING_DAC_TEL_TCU_MODE_VOL_ADDRESS 0x37
#define CONFIG_SETTING_MIC_GAIN_ADDRESS 0x38
#define CONFIG_SETTING_MIC_BIAS_ADDRESS 0x39
#define CONFIG_SETTING_TEL_VOL_ADDRESS 0x3A
#define CONFIG_SETTING_MIC_PREAMP_ADDRESS 0x3B
#define CONFIG_SETTING_TEL_MODE_ADDRESS 0x3C
/* Config 0x40 - 0x50: Audio Settings */
#define CONFIG_SETTING_AUTOPLAY_ADDRESS 0x40
#define CONFIG_SETTING_DAC_AUDIO_VOL_ADDRESS 0x41
#define CONFIG_SETTING_DSP_INPUT_SRC_ADDRESS 0x42
#define CONFIG_SETTING_MANAGE_VOLUME_ADDRESS 0x43
#define CONFIG_SETTING_VOLUME_LOWER_ON_REV_ADDRESS 0x44
/* Config 0x60 - 0x70 */
#define CONFIG_SETTING_LOG_ADDRESS 0x60
#define CONFIG_SETTING_AUTO_POWEROFF_ADDRESS 0x61
#define CONFIG_SETTING_IGN_ALWAYS_ON_ADDRESS 0x62
#define CONFIG_SETTING_SELF_PLAY_ADDRESS 0x63
#define CONFIG_SETTING_LAST_CONNECTED_DEVICE_ADDRESS 0x64
#define CONFIG_SETTING_LAST_CONNECTED_DEVICE_MAC_ADDRESS 0x65 // 0x65 - 0x6A

/* Values 0xA0 - 0xB0: Informational & Counters */
#define CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_MSB_ADDRESS 0xA0
#define CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_LSB_ADDRESS 0xA1

#define CONFIG_DEVICE_LOG_BT 2
#define CONFIG_DEVICE_LOG_IBUS 3
#define CONFIG_DEVICE_LOG_SYSTEM 4
#define CONFIG_DEVICE_LOG_UI 5

#define CONFIG_UI_CD53 1
#define CONFIG_UI_BMBT 2
#define CONFIG_UI_MID 3
#define CONFIG_UI_MID_BMBT 4
#define CONFIG_UI_MIR 5

#define CONFIG_SETTING_OFF 0x00
#define CONFIG_SETTING_ON 0x01
#define CONFIG_SETTING_COMFORT_LOCK_10KM 0x01
#define CONFIG_SETTING_COMFORT_LOCK_20KM 0x02
#define CONFIG_SETTING_COMFORT_UNLOCK_POS_1 0x01
#define CONFIG_SETTING_COMFORT_UNLOCK_POS_0 0x02
#define CONFIG_SETTING_LANGUAGE_AUTO 0x00
#define CONFIG_SETTING_LANGUAGE_DUTCH 0x01
#define CONFIG_SETTING_LANGUAGE_ENGLISH 0x02
#define CONFIG_SETTING_LANGUAGE_FINNISH 0x03
#define CONFIG_SETTING_LANGUAGE_FRENCH 0x04
#define CONFIG_SETTING_LANGUAGE_GERMAN 0x05
#define CONFIG_SETTING_LANGUAGE_NORWEGIAN 0x06
#define CONFIG_SETTING_LANGUAGE_RUSSIAN 0x07
#define CONFIG_SETTING_LANGUAGE_SPANISH 0x08
#define CONFIG_SETTING_LANGUAGE_SWEDISH 0x09
#define CONFIG_SETTING_LANGUAGE_ESTONIAN 0x0A
#define CONFIG_SETTING_LANGUAGE_ITALIAN 0x0B
#define CONFIG_SETTING_LANGUAGE_POLISH 0x0C
#define CONFIG_SETTING_TEMP_COOLANT 0x01
#define CONFIG_SETTING_TEMP_AMBIENT 0x02
#define CONFIG_SETTING_TEMP_OIL 0x03
#define CONFIG_SETTING_TEMP_CELSIUS 0x00
#define CONFIG_SETTING_TEMP_FAHRENHEIT 0x01
#define CONFIG_SETTING_DSP_INPUT_SPDIF 0x01
#define CONFIG_SETTING_DSP_INPUT_ANALOG 0x02
#define CONFIG_SETTING_TEL_VOL_OFFSET_MAX 0x0F
#define CONFIG_SETTING_TEL_MODE_DEFAULT 0x00
#define CONFIG_SETTING_TEL_MODE_TCU 0x01
#define CONFIG_SETTING_TEL_MODE_NO_MUTE 0x02
#define CONFIG_SETTING_PDC_CLUSTER 0x01
#define CONFIG_SETTING_PDC_RADIO 0x02
#define CONFIG_SETTING_PDC_BOTH 0x03

/* EEPROM 0x1A - 0x50: User Configurable Settings */
/* Config 0x1A - 0x24: UI Settings */
#define CONFIG_SETTING_METADATA_MODE CONFIG_SETTING_METADATA_MODE_ADDRESS
#define CONFIG_SETTING_BMBT_DEFAULT_MENU CONFIG_SETTING_BMBT_DEFAULT_MENU_ADDRESS
#define CONFIG_SETTING_BMBT_DASHBOARD_OBC CONFIG_SETTING_BMBT_DASHBOARD_OBC_ADDRESS
#define CONFIG_SETTING_BMBT_TEMP_DISPLAY CONFIG_SETTING_BMBT_TEMP_DISPLAY_ADDRESS
#define CONFIG_SETTING_LANGUAGE CONFIG_SETTING_LANGUAGE_ADDRESS
#define CONFIG_SETTING_MONITOR_OFF CONFIG_SETTING_MONITOR_OFF_ADDRESS
#define CONFIG_SETTING_BMBT_DIST_UNIT CONFIG_SETTING_BMBT_DIST_UNIT_ADDRESS
/* Config 0x25 - 0x35: Comfort Settings */
#define CONFIG_SETTING_COMFORT_BLINKERS CONFIG_SETTING_COMFORT_BLINKERS_ADDRESS
#define CONFIG_SETTING_COMFORT_LOCKS CONFIG_SETTING_COMFORT_LOCKS_ADDRESS
#define CONFIG_SETTING_COMFORT_MIRRORS CONFIG_SETTING_COMFORT_MIRRORS_ADDRESS
#define CONFIG_SETTING_COMFORT_WELCOME_LIGHTS CONFIG_SETTING_COMFORT_WELCOME_LIGHTS_ADDRESS
#define CONFIG_SETTING_COMFORT_HOME_LIGHTS CONFIG_SETTING_COMFORT_HOME_LIGHTS_ADDRESS
#define CONFIG_SETTING_COMFORT_PARKING_LAMPS CONFIG_SETTING_COMFORT_PARKING_LAMPS_ADDRESS
#define CONFIG_SETTING_COMFORT_DATE_TIME CONFIG_SETTING_COMFORT_DATE_TIME_ADDRESS
#define CONFIG_SETTING_COMFORT_AUTOZOOM CONFIG_SETTING_COMFORT_AUTOZOOM_ADDRESS
#define CONFIG_SETTING_COMFORT_PDC CONFIG_SETTING_COMFORT_PDC_ADDRESS
/* Config 0x36 - 0x3E: Telephony Settings */
#define CONFIG_SETTING_HFP CONFIG_SETTING_HFP_ADDRESS
#define CONFIG_SETTING_DAC_TEL_TCU_MODE_VOL CONFIG_SETTING_DAC_TEL_TCU_MODE_VOL_ADDRESS
#define CONFIG_SETTING_MIC_GAIN CONFIG_SETTING_MIC_GAIN_ADDRESS
#define CONFIG_SETTING_MIC_BIAS CONFIG_SETTING_MIC_BIAS_ADDRESS
#define CONFIG_SETTING_TEL_VOL CONFIG_SETTING_TEL_VOL_ADDRESS
#define CONFIG_SETTING_MIC_PREAMP CONFIG_SETTING_MIC_PREAMP_ADDRESS
#define CONFIG_SETTING_TEL_MODE CONFIG_SETTING_TEL_MODE_ADDRESS
/* Config 0x40 - 0x50: Audio Settings */
#define CONFIG_SETTING_AUTOPLAY CONFIG_SETTING_AUTOPLAY_ADDRESS
#define CONFIG_SETTING_DAC_AUDIO_VOL CONFIG_SETTING_DAC_AUDIO_VOL_ADDRESS
#define CONFIG_SETTING_DAC_TEL_VOL CONFIG_SETTING_DAC_TEL_VOL_ADDRESS
#define CONFIG_SETTING_DSP_INPUT_SRC CONFIG_SETTING_DSP_INPUT_SRC_ADDRESS
#define CONFIG_SETTING_MANAGE_VOLUME CONFIG_SETTING_MANAGE_VOLUME_ADDRESS
#define CONFIG_SETTING_VOLUME_LOWER_ON_REV CONFIG_SETTING_VOLUME_LOWER_ON_REV_ADDRESS
/* Config 0x60 - 0x70 */
#define CONFIG_SETTING_LOG CONFIG_SETTING_LOG_ADDRESS
#define CONFIG_SETTING_AUTO_POWEROFF CONFIG_SETTING_AUTO_POWEROFF_ADDRESS
#define CONFIG_SETTING_IGN_ALWAYS_ON CONFIG_SETTING_IGN_ALWAYS_ON_ADDRESS
#define CONFIG_SETTING_SELF_PLAY CONFIG_SETTING_SELF_PLAY_ADDRESS
#define CONFIG_SETTING_LAST_CONNECTED_DEVICE CONFIG_SETTING_LAST_CONNECTED_DEVICE_ADDRESS
#define CONFIG_SETTING_LAST_CONNECTED_DEVICE_MAC CONFIG_SETTING_LAST_CONNECTED_DEVICE_MAC_ADDRESS
/* Values 0xA0 - 0xB0: Informational & Counters */
#define CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_MSB CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_MSB_ADDRESS
#define CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_LSB CONFIG_INFO_BC127_BOOT_FAIL_COUNTER_LSB_ADDRESS
/* Settings Boundary Helpers */
#define CONFIG_SETTING_START_ADDRESS CONFIG_UI_MODE_ADDRESS
#define CONFIG_SETTING_END_ADDRESS 0x70
#define CONFIG_VALUE_START_ADDRESS 0xA0
#define CONFIG_VALUE_END_ADDRESS 0xB0

#define CONFIG_SETTING_CACHE_SIZE CONFIG_SETTING_END_ADDRESS
#define CONFIG_VALUE_CACHE_SIZE (CONFIG_VALUE_END_ADDRESS - CONFIG_VALUE_START_ADDRESS + 1)

uint16_t ConfigGetBC127BootFailures();
uint8_t ConfigGetBuildWeek();
uint8_t ConfigGetBuildYear();
void ConfigGetBytes(uint8_t, uint8_t *, uint8_t);
uint8_t ConfigGetComfortLock();
uint8_t ConfigGetComfortUnlock();
uint8_t ConfigGetFirmwareVersionMajor();
uint8_t ConfigGetFirmwareVersionMinor();
uint8_t ConfigGetFirmwareVersionPatch();
void ConfigGetFirmwareVersionString(char *);
uint8_t ConfigGetIKEType();
uint8_t ConfigGetLightingFeaturesActive();
uint8_t ConfigGetLMVariant();
uint8_t ConfigGetLog(uint8_t);
uint8_t ConfigGetNavType();
uint16_t ConfigGetSerialNumber();
uint8_t ConfigGetSetting(uint8_t);
uint8_t ConfigGetTelephonyFeaturesActive();
uint8_t ConfigGetTempDisplay();
uint8_t ConfigGetTempUnit();
uint8_t ConfigGetDistUnit();
uint8_t ConfigGetTrapCount(uint8_t);
uint8_t ConfigGetTrapLast();
uint8_t ConfigGetUIMode();
uint8_t ConfigGetValue(uint8_t);
uint8_t ConfigGetVehicleType();
void ConfigGetVehicleIdentity(uint8_t *);
void ConfigGetString(uint8_t, char *, uint8_t);
void ConfigSetBC127BootFailures(uint16_t);
void ConfigSetBootloaderMode(uint8_t);
void ConfigSetBytes(uint8_t, const uint8_t *, uint8_t);
void ConfigSetComfortLock(uint8_t);
void ConfigSetComfortUnlock(uint8_t);
void ConfigSetFirmwareVersion(uint8_t, uint8_t, uint8_t);
void ConfigSetIKEType(uint8_t);
void ConfigSetLMVariant(uint8_t);
void ConfigSetLog(uint8_t, uint8_t);
void ConfigSetSetting(uint8_t, uint8_t);
void ConfigSetString(uint8_t, char *, uint8_t);
void ConfigSetNavType(uint8_t);
void ConfigSetTempDisplay(uint8_t);
void ConfigSetTempUnit(uint8_t);
void ConfigSetDistUnit(uint8_t);
void ConfigSetTrapCount(uint8_t, uint8_t);
void ConfigSetTrapIncrement(uint8_t);
void ConfigSetTrapLast(uint8_t);
void ConfigSetUIMode(uint8_t);
void ConfigSetValue(uint8_t, uint8_t);
void ConfigSetVehicleType(uint8_t);
void ConfigSetVehicleIdentity(uint8_t *);
#endif /* CONFIG_H */
